const { rollup } = require('../../util/contracts');
const { applicationData, matrixPools } = require('../../util/db');
const read = require('../../util/forum/read');
const { initiateMatrixPool } = require('./matrix-pools/initiate');

let batchItems;

const initializeBatchItems = async () => {
  try {
    batchItems = await applicationData.get('batchItems');
  } catch (e) {
    batchItems = [];
  }
};

const getBatchItems = () => batchItems;

const addBatchItem = async (postId) => {
  batchItems.push(postId);
  await applicationData.put('batchItems', batchItems);
};

const clearBatchItems = async (itemsToClear) => {
  batchItems = batchItems.filter((item) => !itemsToClear.includes(item));
  await applicationData.put('batchItems', batchItems);
};

const resetBatchItems = async () => {
  batchItems = [];
  // Read from Rollup.items
  const itemCount = await rollup.itemCount();
  const promises = [];
  for (let i = 0; i < itemCount; i += 1) {
    promises.push(rollup.items(i));
  }
  const batchItemsInfo = await Promise.all(promises);
  batchItems = batchItemsInfo.map((x) => x.postId);
  await applicationData.put('batchItems', batchItems);

  // Make sure there's a matrix pool for each batch item.
  // If there's not, then let's start one.
  await Promise.each(batchItemsInfo, async ({ postId, sender, fee }) => {
    let post;
    try {
      post = await read(postId);
    } catch (e) {
      console.error(`Post ID ${postId} not found`);
      return;
    }
    try {
      await matrixPools.get(postId);
    } catch (e) {
      await initiateMatrixPool(postId, post, sender, fee);
    }
  });
  return batchItems;
};

module.exports = {
  initializeBatchItems,
  getBatchItems,
  addBatchItem,
  clearBatchItems,
  resetBatchItems,
};
