const { sendMatrixEvent } = require('../../matrix-bot');
const callWithRetry = require('../../util/call-with-retry');
const {
  rollup, wallet, dao,
  work2,
} = require('../../util/contracts');
const { matrixPools } = require('../../util/db');
const { availabilityStakeDuration } = require('./config');

const stakeRollupAvailability = async () => {
  const currentRep = await dao.balanceOf(await wallet.getAddress());
  if (currentRep) {
    await callWithRetry(() => dao.stakeAvailability(
      rollup.target,
      currentRep,
      availabilityStakeDuration,
    ));
  }
};

const authorsMatch = async (authors, expectedAuthors) => {
  if (expectedAuthors.length !== authors.length) return false;
  return authors.every(({ authorAddress, weightPPM }) => {
    const expectedAuthor = expectedAuthors.find((x) => x.authorAddress === authorAddress);
    return weightPPM === expectedAuthor.weightPPM;
  });
};

const validateWorkEvidence = async (sender, post) => {
  let valid = false;
  if (sender === work2.target) {
    const expectedContent = 'This is a work evidence post';
    valid = post.content.startsWith(expectedContent);
  }
  console.log(`Work evidence ${valid ? 'matched' : 'did not match'} the expected content`);
  return valid;
};

const validatePost = async ({
  sender, post, postId, roomId, eventId, ...params
}) => {
  const currentRep = Number(await dao.balanceOf(await wallet.getAddress()));
  const valid = await validateWorkEvidence(sender, post);
  const stake = { amount: currentRep, account: await wallet.getAddress(), inFavor: valid };
  sendMatrixEvent('io.dgov.pool.stake', { postId, amount: currentRep, inFavor: valid });
  const matrixPool = {
    postId,
    roomId,
    eventId,
    ...params,
    stakes: [stake],
  };
  console.log('matrixPool', matrixPool);
  await matrixPools.put(postId, matrixPool);
};

module.exports = {
  stakeRollupAvailability,
  authorsMatch,
  validateWorkEvidence,
  validatePost,
};
